"""Template tags for internal use within the projectroles app"""

from math import ceil

from django import template
from django.conf import settings
from django.urls import reverse
from django.utils import timezone

from projectroles.models import (
    RoleAssignment,
    RemoteProject,
    SODAR_CONSTANTS,
    PROJECT_TAG_STARRED,
)
from projectroles.plugins import get_active_plugins
from projectroles.project_tags import get_tag_state


register = template.Library()


# SODAR constants
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']
PROJECT_ROLE_OWNER = SODAR_CONSTANTS['PROJECT_ROLE_OWNER']
REMOTE_LEVEL_NONE = SODAR_CONSTANTS['REMOTE_LEVEL_NONE']
REMOTE_LEVEL_REVOKED = SODAR_CONSTANTS['REMOTE_LEVEL_REVOKED']

# Local constants
# Behaviour for certain levels has not been specified/implemented yet
ACTIVE_LEVEL_TYPES = [
    SODAR_CONSTANTS['REMOTE_LEVEL_NONE'],
    SODAR_CONSTANTS['REMOTE_LEVEL_READ_ROLES'],
]
SIDEBAR_ICON_MIN_SIZE = 18
SIDEBAR_ICON_MAX_SIZE = 42


# SODAR and site operations ----------------------------------------------------


@register.simple_tag
def sodar_constant(value):
    """Get value from SODAR_CONSTANTS"""
    return SODAR_CONSTANTS[value] if value in SODAR_CONSTANTS else None


# TODO: Refactor into get_plugins(type)
@register.simple_tag
def get_backend_plugins():
    """Get active backend plugins"""
    return get_active_plugins('backend')


@register.simple_tag
def get_site_app_messages(user):
    """Get messages from site apps"""
    plugins = get_active_plugins('site_app')
    ret = []
    for p in plugins:
        ret += p.get_messages(user)
    return ret


@register.simple_tag
def has_star(project, user):
    """Return True/False for project star tag state"""
    return user.has_perm(
        'projectroles.view_project', project
    ) and get_tag_state(project, user, PROJECT_TAG_STARRED)


@register.simple_tag
def get_remote_project_obj(site, project):
    """Return RemoteProject object for RemoteSite and Project"""
    try:
        return RemoteProject.objects.get(
            site=site, project_uuid=project.sodar_uuid
        )
    except RemoteProject.DoesNotExist:
        return None


@register.simple_tag
def allow_project_creation():
    """Check whether creating a project is allowed on the site"""
    if (
        settings.PROJECTROLES_SITE_MODE == SODAR_CONSTANTS['SITE_MODE_TARGET']
        and not settings.PROJECTROLES_TARGET_CREATE
    ):
        return False
    return True


@register.simple_tag
def is_app_visible(plugin, project, user):
    """Check if app should be visible for user in a specific project"""
    can_view_app = user.has_perm(plugin.app_permission, project)
    app_hidden = False
    if (
        hasattr(settings, 'PROJECTROLES_HIDE_APP_LINKS')
        and plugin.name in settings.PROJECTROLES_HIDE_APP_LINKS
        and not user.is_superuser
    ):
        app_hidden = True
    if (
        can_view_app
        and not app_hidden
        and (project.type == PROJECT_TYPE_PROJECT or plugin.category_enable)
    ):
        return True
    return False


# Template rendering -----------------------------------------------------------


@register.simple_tag
def is_inherited_owner(project, user):
    """Return True if user is inherited owner"""
    if (
        project
        and user
        and user in [r.user for r in project.get_owners(inherited_only=True)]
    ):
        return True
    return False


@register.simple_tag
def get_app_link_state(app_plugin, app_name, url_name):
    """
    Return "active" if plugin matches app_name and url_name is found in
    app_plugin.urls.
    """
    if app_name == app_plugin.name and url_name in [
        u.name for u in app_plugin.urls
    ]:
        return 'active'
    return ''


@register.simple_tag
def get_pr_link_state(app_urls, request, link_names=None):
    """
    Version of get_app_link_state() to be used within the projectroles app.
    If link_names is set, only return "active" if url_name is found in
    link_names.
    """
    if request.resolver_match.app_name != 'projectroles':
        return ''
    url_name = request.resolver_match.url_name
    if url_name in [u.name for u in app_urls]:
        if link_names:
            if not isinstance(link_names, list):
                link_names = [link_names]
            if url_name not in link_names:
                return ''
        return 'active'
    return ''


@register.simple_tag
def get_help_highlight(user):
    """
    Return classes to highlight navbar help link if user has recently signed in
    """
    if user.__class__.__name__ == 'User' and user.is_authenticated:
        delta_days = (timezone.now() - user.date_joined).days
        highlight = getattr(settings, 'PROJECTROLES_HELP_HIGHLIGHT_DAYS', 7)
        if delta_days < highlight:
            return 'font-weight-bold text-warning'
    return ''


@register.simple_tag
def get_role_import_action(source_as, dest_project):
    """Return label for role import action based on existing assignment"""
    try:
        target_as = RoleAssignment.objects.get(
            project=dest_project, user=source_as.user
        )
        if target_as.role == source_as.role:
            return 'No action'
        return 'Update'
    except RoleAssignment.DoesNotExist:
        return 'Import'


@register.simple_tag
def get_login_info():
    """Return HTML info for the login page"""
    ret = '<p>Please log in'

    if getattr(settings, 'ENABLE_LDAP', False):
        ret += ' using your ' + settings.AUTH_LDAP_DOMAIN_PRINTABLE
        if (
            getattr(settings, 'ENABLE_LDAP_SECONDARY', False)
            and settings.AUTH_LDAP2_DOMAIN_PRINTABLE
        ):
            ret += ' or ' + settings.AUTH_LDAP2_DOMAIN_PRINTABLE
        ret += (
            ' account. Enter your user name as <code>username@{}'
            '</code>'.format(settings.AUTH_LDAP_USERNAME_DOMAIN)
        )
        if (
            settings.ENABLE_LDAP_SECONDARY
            and settings.AUTH_LDAP2_USERNAME_DOMAIN
        ):
            ret += ' or <code>username@{}</code>'.format(
                settings.AUTH_LDAP2_USERNAME_DOMAIN
            )
        if getattr(settings, 'PROJECTROLES_ALLOW_LOCAL_USERS', False):
            ret += (
                '. To access the site with local account enter your user '
                'name as <code>username</code>'
            )

    ret += '.</p>'
    return ret


@register.simple_tag
def get_target_project_select(site, project):
    """Return remote target project level selection HTML"""
    current_level = None

    try:
        rp = RemoteProject.objects.get(
            site__mode=SODAR_CONSTANTS['SITE_MODE_TARGET'],
            site=site,
            project_uuid=project.sodar_uuid,
        )
        current_level = rp.level
    except RemoteProject.DoesNotExist:
        pass

    ret = (
        '<select class="form-control form-control-sm" '
        'name="remote_access_{project}" '
        'id="sodar-pr-remote-project-select-{project}">'
        '\n'.format(project=project.sodar_uuid)
    )

    for level in ACTIVE_LEVEL_TYPES:
        selected = False
        if (
            level == REMOTE_LEVEL_NONE
            and current_level
            and current_level != REMOTE_LEVEL_NONE
        ):
            legend = SODAR_CONSTANTS['REMOTE_ACCESS_LEVELS'][
                REMOTE_LEVEL_REVOKED
            ]
            level_val = REMOTE_LEVEL_REVOKED
        else:
            legend = SODAR_CONSTANTS['REMOTE_ACCESS_LEVELS'][level]
            level_val = level

        if level == current_level or (
            level == SODAR_CONSTANTS['REMOTE_LEVEL_NONE'] and not current_level
        ):
            selected = True

        ret += '<option value="{}" {}>{}</option>\n'.format(
            level_val, 'selected' if selected else '', legend
        )

    ret += '</select>\n'
    return ret


@register.simple_tag
def get_remote_access_legend(level):
    """Return legend text for remote project access level"""
    if level not in SODAR_CONSTANTS['REMOTE_ACCESS_LEVELS']:
        return 'N/A'
    return SODAR_CONSTANTS['REMOTE_ACCESS_LEVELS'][level]


@register.simple_tag
def get_sidebar_app_legend(title):
    """Return sidebar link legend HTML"""
    return '<br />'.join(title.split(' '))


@register.simple_tag
def get_sidebar_icon_size():
    """Return sidebar icon size with a min/max limit"""
    return sorted(
        [
            SIDEBAR_ICON_MIN_SIZE,
            getattr(settings, 'PROJECTROLES_SIDEBAR_ICON_SIZE', 32),
            SIDEBAR_ICON_MAX_SIZE,
        ]
    )[1]


@register.simple_tag
def get_sidebar_notch_pos():
    """Return sidebar notch position"""
    return ceil(get_sidebar_icon_size() / 3)


@register.simple_tag
def get_sidebar_notch_size():
    """Return sidebar notch size"""
    return min(ceil(get_sidebar_icon_size() / 2), 12)


@register.simple_tag
def get_sidebar_padding():
    """Return sidebar padding"""
    return ceil(get_sidebar_icon_size() / 4.5)


@register.simple_tag
def get_admin_warning():
    """Return Django admin warning HTML"""
    ret = (
        '<p class="text-danger">Editing database objects other than users '
        'directly in the Django admin view is <strong>not</strong> '
        'recommended!</p>'
    )
    ret += (
        '<p class="text-danger">Actions taken in the admin view may '
        'result in system malfunction or data loss! Please proceed with '
        'caution.</p>'
    )
    ret += (
        '<p><a class="btn btn-danger pull-right" role="button" '
        'target="_blank" href="{}">'
        '<i class="iconify" data-icon="mdi:cogs"></i> Continue to Django Admin'
        '</a></p>'.format(reverse('admin:index'))
    )
    return ret



"""Tests for Ajax view permissions in the adminalerts app"""

from django.test import override_settings
from django.urls import reverse

# Projectroles dependency
from projectroles.tests.test_permissions import TestPermissionBase

from adminalerts.tests.test_models import AdminAlertMixin


class TestAdminAlertPermissions(AdminAlertMixin, TestPermissionBase):
    """Tests for AdminAlert views"""

    def setUp(self):
        # Create users
        self.superuser = self.make_user('superuser')
        self.superuser.is_superuser = True
        self.superuser.is_staff = True
        self.superuser.save()

        self.regular_user = self.make_user('regular_user')

        # No user
        self.anonymous = None

        # Create alert
        self.alert = self._make_alert(
            message='alert',
            user=self.superuser,
            description='description',
            active=True,
        )

    def test_active_toggle(self):
        """Test permissions for AdminAlert activation API view"""
        url = reverse(
            'adminalerts:ajax_active_toggle',
            kwargs={'adminalert': self.alert.sodar_uuid},
        )
        good_users = [self.superuser]
        bad_users = [self.anonymous, self.regular_user]
        self.assert_response(url, good_users, 200, method='POST')
        self.assert_response(url, bad_users, 403, method='POST')

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_active_toggle_anon(self):
        """Test permissions for AdminAlert activation API view with anonymous access"""
        url = reverse(
            'adminalerts:ajax_active_toggle',
            kwargs={'adminalert': self.alert.sodar_uuid},
        )
        self.assert_response(url, self.anonymous, 403, method='POST')



"""Ajax API view permission tests for the timeline app"""

from django.test import override_settings
from django.urls import reverse

# Projectroles dependency
from projectroles.tests.test_permissions import TestProjectPermissionBase

from timeline.tests.test_models import (
    ProjectEventMixin,
    ProjectEventStatusMixin,
)


class TestTimelineAjaxPermissions(
    ProjectEventMixin, ProjectEventStatusMixin, TestProjectPermissionBase
):
    """Tests for timeline Ajax API view permissions"""

    def setUp(self):
        super().setUp()
        self.event = self._make_event(
            self.project, 'projectroles', self.user_owner, 'project_create'
        )
        self._make_event_status(self.event, 'OK')
        self.site_event = self._make_event(
            None, 'projectroles', self.user_owner, 'test_event'
        )
        self._make_event_status(self.site_event, 'OK')
        self.regular_user = self.make_user('regular_user')

    def test_detail_project(self):
        """Test ProjectEventDetailAjaxView permissions"""
        url = reverse(
            'timeline:ajax_detail_project',
            kwargs={'projectevent': self.event.sodar_uuid},
        )
        good_users = [
            self.superuser,
            self.owner_as.user,
            self.delegate_as.user,
            self.contributor_as.user,
            self.guest_as.user,
        ]
        bad_users = [
            self.user_no_roles,
            self.anonymous,
        ]
        self.assert_response(url, good_users, 200)
        self.assert_response(url, bad_users, 403)
        self.project.set_public()
        self.assert_response(url, self.anonymous, 403)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_detail_project_anon(self):
        """Test ProjectEventDetailAjaxView permissions with anonymous access"""
        url = reverse(
            'timeline:ajax_detail_project',
            kwargs={'projectevent': self.event.sodar_uuid},
        )
        self.assert_response(url, self.anonymous, 403)
        self.project.set_public()
        self.assert_response(url, self.anonymous, 200)

    def test_detail_project_classified(self):
        """Test ProjectEventDetailAjaxView permissions with classified event"""
        self.event.classified = True
        self.event.save()
        url = reverse(
            'timeline:ajax_detail_project',
            kwargs={'projectevent': self.event.sodar_uuid},
        )
        good_users = [
            self.superuser,
            self.owner_as.user,
            self.delegate_as.user,
        ]
        bad_users = [
            self.contributor_as.user,
            self.guest_as.user,
            self.user_no_roles,
            self.anonymous,
        ]
        self.assert_response(url, good_users, 200)
        self.assert_response(url, bad_users, 403)
        self.project.set_public()
        self.assert_response(url, self.anonymous, 403)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_detail_project_classified_anon(self):
        """Test ProjectEventDetailAjaxView permissions with classified event and anonymous access"""
        self.event.classified = True
        self.event.save()
        url = reverse(
            'timeline:ajax_detail_project',
            kwargs={'projectevent': self.event.sodar_uuid},
        )
        self.assert_response(url, self.anonymous, 403)
        self.project.set_public()
        self.assert_response(url, self.anonymous, 403)

    def test_detail_site(self):
        """Test SiteEventDetailAjaxView permissions"""
        url = reverse(
            'timeline:ajax_detail_site',
            kwargs={'projectevent': self.site_event.sodar_uuid},
        )
        good_users = [
            self.superuser,
            self.regular_user,
        ]
        self.assert_response(url, good_users, 200)
        self.assert_response(url, self.anonymous, 403)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_detail_site_anon(self):
        """Test SiteEventDetailAjaxView permissions with anonymous access"""
        url = reverse(
            'timeline:ajax_detail_site',
            kwargs={'projectevent': self.site_event.sodar_uuid},
        )
        self.assert_response(url, self.anonymous, 403)

    def test_detail_site_classified(self):
        """Test SiteEventDetailAjaxView permissions with classified event"""
        self.event.classified = True
        self.event.save()
        url = reverse(
            'timeline:ajax_detail_site',
            kwargs={'projectevent': self.site_event.sodar_uuid},
        )
        good_users = [
            self.superuser,
            self.regular_user,
        ]
        self.assert_response(url, good_users, 200)
        self.assert_response(url, self.anonymous, 403)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_detail_site_classified_anon(self):
        """Test SiteEventDetailAjaxView permissions with classified event and anonymous access"""
        self.event.classified = True
        self.event.save()
        url = reverse(
            'timeline:ajax_detail_site',
            kwargs={'projectevent': self.site_event.sodar_uuid},
        )
        self.assert_response(url, self.anonymous, 403)



from django.conf.urls import url

from . import views, views_api

# NOTE: file/folder/hyperlink objects can be referred to as 'item', but only if
#       ObjectPermissionMixin is used in the view


app_name = 'filesfolders'

urls_ui = [
    url(
        regex=r'^(?P<project>[0-9a-f-]+)$',
        view=views.ProjectFileView.as_view(),
        name='list',
    ),
    url(
        regex=r'^folder/(?P<folder>[0-9a-f-]+)$',
        view=views.ProjectFileView.as_view(),
        name='list',
    ),
    url(
        regex=r'^upload/(?P<project>[0-9a-f-]+)$',
        view=views.FileCreateView.as_view(),
        name='file_create',
    ),
    url(
        regex=r'^upload/in/(?P<folder>[0-9a-f-]+)$',
        view=views.FileCreateView.as_view(),
        name='file_create',
    ),
    url(
        regex=r'^update/(?P<item>[0-9a-f-]+)$',
        view=views.FileUpdateView.as_view(),
        name='file_update',
    ),
    url(
        regex=r'^delete/(?P<item>[0-9a-f-]+)$',
        view=views.FileDeleteView.as_view(),
        name='file_delete',
    ),
    url(
        regex=r'^download/(?P<file>[0-9a-f-]+)/(?P<file_name>[^\0/]+)$',
        view=views.FileServeView.as_view(),
        name='file_serve',
    ),
    url(
        regex=r'^download/(?P<secret>[\w\-]+)/(?P<file_name>[^\0/]+)$',
        view=views.FileServePublicView.as_view(),
        name='file_serve_public',
    ),
    url(
        regex=r'^link/(?P<file>[0-9a-f-]+)$',
        view=views.FilePublicLinkView.as_view(),
        name='file_public_link',
    ),
    url(
        regex=r'^folder/add/(?P<project>[0-9a-f-]+)$',
        view=views.FolderCreateView.as_view(),
        name='folder_create',
    ),
    url(
        regex=r'^folder/add/in/(?P<folder>[0-9a-f-]+)$',
        view=views.FolderCreateView.as_view(),
        name='folder_create',
    ),
    url(
        regex=r'^folder/update/(?P<item>[0-9a-f-]+)$',
        view=views.FolderUpdateView.as_view(),
        name='folder_update',
    ),
    url(
        regex=r'^folder/delete(?P<item>[0-9a-f-]+)$',
        view=views.FolderDeleteView.as_view(),
        name='folder_delete',
    ),
    url(
        regex=r'^link/add/(?P<project>[0-9a-f-]+)$',
        view=views.HyperLinkCreateView.as_view(),
        name='hyperlink_create',
    ),
    url(
        regex=r'^link/add/in/(?P<folder>[0-9a-f-]+)$',
        view=views.HyperLinkCreateView.as_view(),
        name='hyperlink_create',
    ),
    url(
        regex=r'^link/update/(?P<item>[0-9a-f-]+)$',
        view=views.HyperLinkUpdateView.as_view(),
        name='hyperlink_update',
    ),
    url(
        regex=r'^link/delete/(?P<item>[0-9a-f-]+)$',
        view=views.HyperLinkDeleteView.as_view(),
        name='hyperlink_delete',
    ),
    url(
        regex=r'^batch/(?P<project>[0-9a-f-]+)$',
        view=views.BatchEditView.as_view(),
        name='batch_edit',
    ),
    url(
        regex=r'^batch/in/(?P<folder>[0-9a-f-]+)$',
        view=views.BatchEditView.as_view(),
        name='batch_edit',
    ),
]

urls_api = [
    url(
        regex=r'^api/folder/list-create/(?P<project>[0-9a-f-]+)$',
        view=views_api.FolderListCreateAPIView.as_view(),
        name='api_folder_list_create',
    ),
    url(
        regex=r'^api/folder/retrieve-update-destroy/(?P<folder>[0-9a-f-]+)$',
        view=views_api.FolderRetrieveUpdateDestroyAPIView.as_view(),
        name='api_folder_retrieve_update_destroy',
    ),
    url(
        regex=r'^api/file/list-create/(?P<project>[0-9a-f-]+)$',
        view=views_api.FileListCreateAPIView.as_view(),
        name='api_file_list_create',
    ),
    url(
        regex=r'^api/file/retrieve-update-destroy/(?P<file>[0-9a-f-]+)$',
        view=views_api.FileRetrieveUpdateDestroyAPIView.as_view(),
        name='api_file_retrieve_update_destroy',
    ),
    url(
        regex=r'^api/file/serve/(?P<file>[0-9a-f-]+)$',
        view=views_api.FileServeAPIView.as_view(),
        name='api_file_serve',
    ),
    url(
        regex=r'^api/hyperlink/list-create/(?P<project>[0-9a-f-]+)$',
        view=views_api.HyperLinkListCreateAPIView.as_view(),
        name='api_hyperlink_list_create',
    ),
    url(
        regex=r'^api/hyperlink/retrieve-update-destroy/(?P<hyperlink>[0-9a-f-]+)$',
        view=views_api.HyperLinkRetrieveUpdateDestroyAPIView.as_view(),
        name='api_hyperlink_retrieve_update_destroy',
    ),
]

urlpatterns = urls_ui + urls_api



"""Taskflow view tests for the projectroles app"""

import os
from tempfile import NamedTemporaryFile

from unittest import skipIf

from projectroles.management.commands.batchupdateroles import (
    Command as BatchUpdateRolesCommand,
)
from projectroles.models import (
    RoleAssignment,
    ProjectInvite,
    SODAR_CONSTANTS,
)
from projectroles.tests.test_commands import BatchUpdateRolesMixin
from projectroles.tests.test_views_taskflow import (
    TestTaskflowBase,
    TASKFLOW_ENABLED,
    TASKFLOW_SKIP_MSG,
)

# SODAR constants
PROJECT_ROLE_OWNER = SODAR_CONSTANTS['PROJECT_ROLE_OWNER']
PROJECT_ROLE_DELEGATE = SODAR_CONSTANTS['PROJECT_ROLE_DELEGATE']
PROJECT_ROLE_CONTRIBUTOR = SODAR_CONSTANTS['PROJECT_ROLE_CONTRIBUTOR']
PROJECT_ROLE_GUEST = SODAR_CONSTANTS['PROJECT_ROLE_GUEST']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']


@skipIf(not TASKFLOW_ENABLED, TASKFLOW_SKIP_MSG)
class TestBatchUpdateRoles(BatchUpdateRolesMixin, TestTaskflowBase):
    """Tests for batchupdateroles command"""

    def setUp(self):
        super().setUp()

        # Make project with owner in Taskflow and Django
        self.project, self.owner_as = self._make_project_taskflow(
            title='TestProject',
            type=PROJECT_TYPE_PROJECT,
            parent=self.category,
            owner=self.user,
            description='description',
        )

        # Init command class
        self.command = BatchUpdateRolesCommand(sodar_url=self.get_sodar_url())
        # Init file
        self.file = NamedTemporaryFile(delete=False)

    def tearDown(self):
        if self.file:
            os.remove(self.file.name)
        super().tearDown()

    def test_role_update(self):
        """Test updating an existing role for user with taskflow"""
        p_uuid = str(self.project.sodar_uuid)
        email = 'new@example.com'
        user_new = self.make_user('user_new')
        user_new.email = email
        user_new.save()
        role_as = self._make_assignment_taskflow(
            self.project, user_new, self.role_guest
        )

        # Assert preconditions
        self.assertEqual(
            RoleAssignment.objects.filter(
                project=self.project, user=user_new
            ).count(),
            1,
        )

        self._write_file([p_uuid, email, PROJECT_ROLE_CONTRIBUTOR])
        self.command.handle(
            **{'file': self.file.name, 'issuer': self.user.username}
        )

        # Assert postconditions
        self.assertEqual(ProjectInvite.objects.count(), 0)
        role_as.refresh_from_db()
        self.assertEqual(role_as.role, self.role_contributor)



"""Ajax API view tests for the projectroles app"""

import json

from django.forms import model_to_dict
from django.test import override_settings
from django.urls import reverse

from projectroles.models import ProjectUserTag, PROJECT_TAG_STARRED
from projectroles.tests.test_models import (
    ProjectMixin,
    RoleAssignmentMixin,
    ProjectUserTagMixin,
)
from projectroles.tests.test_views import (
    TestViewsBase,
    PROJECT_TYPE_CATEGORY,
    PROJECT_TYPE_PROJECT,
)
from projectroles.views_ajax import INHERITED_OWNER_INFO


class TestProjectListAjaxView(ProjectMixin, RoleAssignmentMixin, TestViewsBase):
    """Tests for ProjectListAjaxView"""

    def setUp(self):
        super().setUp()
        self.category = self._make_project(
            'TestCategory', PROJECT_TYPE_CATEGORY, None
        )
        self.owner_as_cat = self._make_assignment(
            self.category, self.user, self.role_owner
        )
        self.project = self._make_project(
            'TestProject', PROJECT_TYPE_PROJECT, self.category
        )
        self.owner_as = self._make_assignment(
            self.project, self.user, self.role_owner
        )

    def test_get(self):
        """Test project list retrieval"""
        with self.login(self.user):
            response = self.client.get(
                reverse('projectroles:ajax_project_list'),
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            'projects': [
                {
                    'title': self.category.title,
                    'type': self.category.type,
                    'full_title': self.category.full_title,
                    'public_guest_access': self.category.public_guest_access,
                    'remote': False,
                    'revoked': False,
                    'starred': False,
                    'depth': 0,
                    'uuid': str(self.category.sodar_uuid),
                },
                {
                    'title': self.project.title,
                    'type': self.project.type,
                    'full_title': self.project.full_title,
                    'public_guest_access': self.project.public_guest_access,
                    'remote': False,
                    'revoked': False,
                    'starred': False,
                    'depth': 1,
                    'uuid': str(self.project.sodar_uuid),
                },
            ],
            'parent_depth': 0,
            'messages': {},
            'user': {'superuser': True},
        }
        self.assertEqual(response.data, expected)

    def test_get_parent(self):
        """Test project list retrieval with a parent project"""
        with self.login(self.user):
            response = self.client.get(
                reverse('projectroles:ajax_project_list')
                + '?parent='
                + str(self.category.sodar_uuid),
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            'projects': [
                {
                    'title': self.project.title,
                    'type': self.project.type,
                    'full_title': self.project.title,  # Not full_title
                    'public_guest_access': self.project.public_guest_access,
                    'remote': False,
                    'revoked': False,
                    'starred': False,
                    'depth': 1,
                    'uuid': str(self.project.sodar_uuid),
                },
            ],
            'parent_depth': 1,
            'messages': {},
            'user': {'superuser': True},
        }
        self.assertEqual(response.data, expected)

    def test_get_inherited_owner(self):
        """Test project list retrieval for inherited owner"""
        cat_owner = self.make_user('cat_owner')
        self.owner_as_cat.user = cat_owner
        self.owner_as_cat.save()
        pro_owner = self.make_user('pro_owner')
        self.owner_as.user = pro_owner
        self.owner_as.save()
        with self.login(cat_owner):
            response = self.client.get(
                reverse('projectroles:ajax_project_list'),
            )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data['projects']), 2)

    def test_get_no_results(self):
        """Test project list retrieval with no results"""
        new_user = self.make_user('new_user')  # User with no roles
        with self.login(new_user):
            response = self.client.get(
                reverse('projectroles:ajax_project_list'),
            )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data['projects'], [])
        self.assertIsNotNone(response.data['messages'].get('no_projects'))

    def test_get_project_parent(self):
        """Test project list retrieval with project as parent (should fail)"""
        with self.login(self.user):
            response = self.client.get(
                reverse('projectroles:ajax_project_list')
                + '?parent='
                + str(self.project.sodar_uuid),
            )
        self.assertEqual(response.status_code, 400)


class TestProjectListColumnAjaxView(
    ProjectMixin, RoleAssignmentMixin, TestViewsBase
):
    """Tests for ProjectListColumnAjaxView"""

    def setUp(self):
        super().setUp()
        self.category = self._make_project(
            'TestCategory', PROJECT_TYPE_CATEGORY, None
        )
        self.owner_as_cat = self._make_assignment(
            self.category, self.user, self.role_owner
        )
        self.project = self._make_project(
            'TestProject', PROJECT_TYPE_PROJECT, self.category
        )
        self.owner_as = self._make_assignment(
            self.project, self.user, self.role_owner
        )

    def test_post(self):
        """Test POST for custom column retrieval"""
        with self.login(self.user):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_columns'),
                json.dumps({'projects': [str(self.project.sodar_uuid)]}),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            str(self.project.sodar_uuid): {
                'filesfolders': {'files': {'html': '0'}, 'links': {'html': '0'}}
            }
        }
        self.assertEqual(response.data, expected)

    @override_settings(FILESFOLDERS_SHOW_LIST_COLUMNS=False)
    def test_post_no_columns(self):
        """Test POST with no custom colums"""
        with self.login(self.user):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_columns'),
                json.dumps({'projects': [str(self.project.sodar_uuid)]}),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        expected = {str(self.project.sodar_uuid): {}}
        self.assertEqual(response.data, expected)

    def test_post_no_permission(self):
        """Test POST with no user permission on a project"""
        new_project = self._make_project(
            'NewProject', PROJECT_TYPE_PROJECT, None
        )
        new_user = self.make_user('new_user')
        self._make_assignment(new_project, new_user, self.role_owner)

        with self.login(new_user):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_columns'),
                json.dumps(
                    {
                        'projects': [
                            str(self.project.sodar_uuid),
                            str(new_project.sodar_uuid),
                        ]
                    }
                ),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            str(new_project.sodar_uuid): {
                'filesfolders': {'files': {'html': '0'}, 'links': {'html': '0'}}
            }
        }
        self.assertEqual(response.data, expected)


class TestProjectListRoleAjaxView(
    ProjectMixin, RoleAssignmentMixin, TestViewsBase
):
    """Tests for ProjectListRoleAjaxView"""

    def setUp(self):
        super().setUp()
        self.user_cat_owner = self.make_user('cat_owner')
        self.user_pro_owner = self.make_user('pro_owner')
        self.category = self._make_project(
            'TestCategory', PROJECT_TYPE_CATEGORY, None
        )
        self.owner_as_cat = self._make_assignment(
            self.category, self.user_cat_owner, self.role_owner
        )
        self.project = self._make_project(
            'TestProject', PROJECT_TYPE_PROJECT, self.category
        )
        self.owner_as = self._make_assignment(
            self.project, self.user_pro_owner, self.role_owner
        )

    def test_post_category_owner(self):
        """Test POST for role retrieval as category owner"""
        with self.login(self.user_cat_owner):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_roles'),
                json.dumps(
                    {
                        'projects': [
                            str(self.category.sodar_uuid),
                            str(self.project.sodar_uuid),
                        ]
                    }
                ),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            str(self.category.sodar_uuid): {
                'name': 'Owner',
                'class': None,
                'info': None,
            },
            str(self.project.sodar_uuid): {
                'name': 'Owner',
                'class': 'text-muted',
                'info': INHERITED_OWNER_INFO,
            },
        }
        self.assertEqual(response.data, expected)

    def test_post_project_owner(self):
        """Test POST for role retrieval as project owner"""
        with self.login(self.user_pro_owner):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_roles'),
                json.dumps(
                    {
                        'projects': [
                            str(self.category.sodar_uuid),
                            str(self.project.sodar_uuid),
                        ]
                    }
                ),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        expected = {
            str(self.category.sodar_uuid): {
                'name': 'N/A',
                'class': 'text-muted',
                'info': None,
            },
            str(self.project.sodar_uuid): {
                'name': 'Owner',
                'class': None,
                'info': None,
            },
        }
        self.assertEqual(response.data, expected)

    def test_post_no_access(self):
        """Test POST for role retrieval with no access to project"""
        new_user = self.make_user('new_user')  # User with no roles
        with self.login(new_user):
            response = self.client.post(
                reverse('projectroles:ajax_project_list_roles'),
                json.dumps(
                    {
                        'projects': [
                            str(self.category.sodar_uuid),
                            str(self.project.sodar_uuid),
                        ]
                    }
                ),
                content_type='application/json',
            )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, {})


class TestProjectStarringAjaxView(
    ProjectMixin, RoleAssignmentMixin, ProjectUserTagMixin, TestViewsBase
):
    """Tests for ProjectStarringAjaxView"""

    def setUp(self):
        super().setUp()
        self.project = self._make_project(
            'TestProject', PROJECT_TYPE_PROJECT, None
        )
        self.owner_as = self._make_assignment(
            self.project, self.user, self.role_owner
        )

    def test_star_project(self):
        """Test project starring"""
        self.assertEqual(ProjectUserTag.objects.all().count(), 0)

        with self.login(self.user):
            response = self.client.post(
                reverse(
                    'projectroles:ajax_star',
                    kwargs={'project': self.project.sodar_uuid},
                )
            )

        self.assertEqual(ProjectUserTag.objects.all().count(), 1)
        tag = ProjectUserTag.objects.get(
            project=self.project, user=self.user, name=PROJECT_TAG_STARRED
        )
        self.assertIsNotNone(tag)
        expected = {
            'id': tag.pk,
            'project': self.project.pk,
            'user': self.user.pk,
            'name': PROJECT_TAG_STARRED,
            'sodar_uuid': tag.sodar_uuid,
        }
        self.assertEqual(model_to_dict(tag), expected)
        self.assertEqual(response.status_code, 200)

    def test_unstar_project(self):
        """Test project unstarring"""
        self._make_tag(self.project, self.user, name=PROJECT_TAG_STARRED)
        self.assertEqual(ProjectUserTag.objects.all().count(), 1)

        with self.login(self.user):
            response = self.client.post(
                reverse(
                    'projectroles:ajax_star',
                    kwargs={'project': self.project.sodar_uuid},
                )
            )

        self.assertEqual(ProjectUserTag.objects.all().count(), 0)
        self.assertEqual(response.status_code, 200)
